package org.xbib.interlibrary.order.soap;

import jakarta.xml.bind.annotation.XmlAccessType;
import jakarta.xml.bind.annotation.XmlAccessorType;
import jakarta.xml.bind.annotation.XmlElement;
import jakarta.xml.bind.annotation.XmlRootElement;
import jakarta.xml.bind.annotation.adapters.XmlAdapter;
import jakarta.xml.bind.annotation.adapters.XmlJavaTypeAdapter;

import java.nio.ByteBuffer;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.logging.Logger;

@XmlRootElement(name = "createorder")
@XmlAccessorType(XmlAccessType.FIELD)
public class CreateOrder {

    private static final Logger logger = Logger.getLogger(CreateOrder.class.getName());

    private static final CharsetDecoder utf8Decoder = StandardCharsets.UTF_8.newDecoder();

    private static final CharsetDecoder iso88591Decoder = StandardCharsets.ISO_8859_1.newDecoder();

    @XmlElement(name="delivery-service")
    //@XmlJavaTypeAdapter(MyBase64Adapter.class)
    public String deliveryService;

    @XmlElement(name="addroute")
    public String addroute;

    @XmlElement(name="delivery-address")
    public DeliveryAddress deliveryAddress;

    @XmlElement(name="transaction-id")
    public TransactionId transactionId;

    @XmlElement(name="requester-note")
    @XmlJavaTypeAdapter(MyBase64Adapter.class)
    public String requesterNote;

    @XmlElement(name="transaction-type")
    @XmlJavaTypeAdapter(MyBase64Adapter.class)
    public String transactionType;

    @XmlElement(name="cost-info-type")
    public CostInfoType costInfoType;

    public CreateOrder() {
    }

    public String getDeliveryService() {
        return deliveryService;
    }

    public String getAddroute() {
        return addroute;
    }

    public DeliveryAddress getDeliveryAddress() {
        return deliveryAddress;
    }

    public TransactionId getTransactionId() {
        return transactionId;
    }

    public String getRequesterNote() {
        return requesterNote;
    }

    public String getTransactionType() {
        return transactionType;
    }

    public CostInfoType getCostInfoType() {
        return costInfoType;
    }

    @XmlAccessorType(XmlAccessType.FIELD)
    @XmlRootElement(name="delivery-address")
    public static class DeliveryAddress {

        @XmlElement(name="city")
        @XmlJavaTypeAdapter(MyBase64Adapter.class)
        public String city;

        @XmlElement(name="post-office-box")
        @XmlJavaTypeAdapter(MyBase64Adapter.class)
        public String postOfficeBox;

        @XmlElement(name="name-of-person-or-institution")
        @XmlJavaTypeAdapter(MyBase64Adapter.class)
        public String nameOfPersonOrInstitution;

        @XmlElement(name="postal-code")
        @XmlJavaTypeAdapter(MyBase64Adapter.class)
        public String postalCode;

        @XmlElement(name="street-and-number")
        @XmlJavaTypeAdapter(MyBase64Adapter.class)
        public String streetAndNumber;

        @XmlElement(name="extended-postal-delivery-address")
        @XmlJavaTypeAdapter(MyBase64Adapter.class)
        public String extendedPostalDeliveryAddress;

        public DeliveryAddress() {
        }

        public String getCity() {
            return city;
        }

        public String getPostOfficeBox() {
            return postOfficeBox;
        }

        public String getNameOfPersonOrInstitution() {
            return nameOfPersonOrInstitution;
        }

        public String getPostalCode() {
            return postalCode;
        }

        public String getStreetAndNumber() {
            return streetAndNumber;
        }

        public String getExtendedPostalDeliveryAddress() {
            return extendedPostalDeliveryAddress;
        }
    }

    @XmlAccessorType(XmlAccessType.FIELD)
    @XmlRootElement(name="transaction-id")
    public static class TransactionId {

        @XmlElement(name="transaction-qualifier")
        public String transactionQualifier;

        public String getTransactionQualifier() {
            return transactionQualifier;
        }
    }

    @XmlAccessorType(XmlAccessType.FIELD)
    @XmlRootElement(name="cost-info-type")
    public static class CostInfoType {

        @XmlElement(name="maximum-cost")
        @XmlJavaTypeAdapter(MyBase64Adapter.class)
        public String maximumCost;

        public String getMaximumCost() {
            return maximumCost;
        }
    }

    private static class MyBase64Adapter extends XmlAdapter<String, String> {

        @Override
        public String unmarshal(String v) {
            if (v == null) {
                return null;
            }
            try {
                return decode(Base64.getMimeDecoder().decode(v));
            } catch (Exception e) {
                return v;
            }
        }

        @Override
        public String marshal(String v) {
            if (v == null) {
                return null;
            }
            return Base64.getMimeEncoder().encodeToString(v.getBytes(StandardCharsets.UTF_8));
        }
    }

    private static String decode(byte[] b) {
        if (b == null) {
            return null;
        }
        try {
            return utf8Decoder.decode(ByteBuffer.wrap(b)).toString();
        } catch (CharacterCodingException e) {
            try {
                return iso88591Decoder.decode(ByteBuffer.wrap(b)).toString();
            } catch (CharacterCodingException ee) {
                return null;
            }
        }
    }
}
