package org.xbib.interlibrary.order.soap;

import jakarta.xml.bind.JAXBContext;
import jakarta.xml.bind.JAXBElement;
import jakarta.xml.bind.JAXBException;
import jakarta.xml.bind.Unmarshaller;
import jakarta.xml.soap.MessageFactory;
import jakarta.xml.soap.MimeHeaders;
import jakarta.xml.soap.SOAPException;
import jakarta.xml.soap.SOAPMessage;
import org.w3c.dom.Document;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Writer;

public class SoapParser {

    private Document document;

    public CreateOrder getResult(InputStream inputStream) throws SOAPException, IOException, JAXBException, TransformerException {
        ByteArrayOutputStream soapOutputStream = new ByteArrayOutputStream();
        removeGensym(inputStream, soapOutputStream);
        InputStream soapInputStream = new ByteArrayInputStream(soapOutputStream.toByteArray());
        MessageFactory messageFactory = MessageFactory.newInstance();
        SOAPMessage message = messageFactory.createMessage(new MimeHeaders(), soapInputStream);
        this.document = message.getSOAPBody().extractContentAsDocument();
        // this must be added to avaid java.lang.IllegalArgumentException: prefix xsd is not bound to a namespace
        document.getDocumentElement().setAttribute("xmlns:xsd", "http://www.w3.org/2001/XMLSchema");
        JAXBContext jaxbContext = JAXBContext.newInstance(CreateOrder.class, CreateOrderObjectFactory.class);
        Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
        JAXBElement<CreateOrder> createOrderJAXBElement = unmarshaller.unmarshal(document, CreateOrder.class);
        return createOrderJAXBElement.getValue();
    }

    public void printDocument(Writer writer) throws TransformerException {
        TransformerFactory tf = TransformerFactory.newInstance();
        Transformer transformer = tf.newTransformer();
        transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
        transformer.setOutputProperty(OutputKeys.METHOD, "xml");
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
        transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
        transformer.transform(new DOMSource(document), new StreamResult(writer));
    }

    private void removeGensym(InputStream inputStream, OutputStream outputStream) throws TransformerException, IOException {
        TransformerFactory tf = TransformerFactory.newInstance();
        try (InputStream removeGensym = getClass().getResourceAsStream("remove-gensym.xslt")) {
            if (removeGensym == null) {
                throw new IOException("remove-gensym.xslt not found");
            }
            Transformer transformer = tf.newTransformer(new StreamSource(removeGensym));
            transformer.transform(new StreamSource(inputStream), new StreamResult(outputStream));
        }
    }
}
